@@ -1,4 +1,4 @@
-# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2018-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,6 +10,7 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Test docstring"""
 from __future__ import absolute_import
 
 import sagemaker
@@ -21,9 +22,10 @@ from sagemaker.predictor import RealTimePredictor
 
 
 class AlgorithmEstimator(EstimatorBase):
-    """A generic Estimator to train using any algorithm object (with an ``algorithm_arn``).
-    The Algorithm can be your own, or any Algorithm from AWS Marketplace that you have a valid
-    subscription for. This class will perform client-side validation on all the inputs.
+    """A generic Estimator to train using any algorithm object (with an
+    ``algorithm_arn``). The Algorithm can be your own, or any Algorithm from AWS
+    Marketplace that you have a valid subscription for. This class will perform
+    client-side validation on all the inputs.
     """
 
     # These Hyperparameter Types have a range definition.
@@ -51,58 +53,80 @@ class AlgorithmEstimator(EstimatorBase):
         model_channel_name="model",
         metric_definitions=None,
         encrypt_inter_container_traffic=False,
+        **kwargs  # pylint: disable=W0613
     ):
         """Initialize an ``AlgorithmEstimator`` instance.
 
         Args:
-           algorithm_arn (str): algorithm arn used for training. Can be just the name if your
+            algorithm_arn (str): algorithm arn used for training. Can be just the name if your
                 account owns the algorithm.
-            role (str): An AWS IAM role (either name or full ARN). The Amazon SageMaker training jobs and APIs
-                that create Amazon SageMaker endpoints use this role to access training data and model artifacts.
-                After the endpoint is created, the inference code might use the IAM role,
-                if it needs to access an AWS resource.
-            train_instance_count (int): Number of Amazon EC2 instances to use for training.
-            train_instance_type (str): Type of EC2 instance to use for training, for example, 'ml.c4.xlarge'.
-            train_volume_size (int): Size in GB of the EBS volume to use for storing input data
-                during training (default: 30). Must be large enough to store training data if File Mode is used
-                (which is the default).
-            train_volume_kms_key (str): Optional. KMS key ID for encrypting EBS volume attached to the
-                training instance (default: None).
+            role (str): An AWS IAM role (either name or full ARN). The Amazon SageMaker
+                training jobs and APIsthat create Amazon SageMaker endpoints use this role to
+                access training data and model artifacts. After the endpoint
+                is created, the inference code might use the IAM role, if it
+                needs to access an AWS resource.
+            train_instance_count (int): Number of Amazon EC2 instances to
+                use for training. train_instance_type (str): Type of EC2
+                instance to use for training, for example, 'ml.c4.xlarge'.
+            train_volume_size (int): Size in GB of the EBS volume to use for
+                storing input data during training (default: 30). Must be large enough to store
+                training data if File Mode is used (which is the default).
+            train_volume_kms_key (str): Optional. KMS key ID for encrypting EBS volume attached
+                to the training instance (default: None).
             train_max_run (int): Timeout in seconds for training (default: 24 * 60 * 60).
-                After this amount of time Amazon SageMaker terminates the job regardless of its current status.
-            input_mode (str): The input mode that the algorithm supports (default: 'File'). Valid modes:
+                After this amount of time Amazon SageMaker terminates the
+                job regardless of its current status.
+            input_mode (str): The input mode that the algorithm supports
+            (default: 'File'). Valid modes:
 
-                * 'File' - Amazon SageMaker copies the training dataset from the S3 location to a local directory.
-                * 'Pipe' - Amazon SageMaker streams data directly from S3 to the container via a Unix-named pipe.
+                * 'File' - Amazon SageMaker copies the training dataset from
+                  the S3 location to a local directory.
+                * 'Pipe' - Amazon SageMaker streams data directly from S3 to
+                  the container via a Unix-named pipe.
 
-                This argument can be overriden on a per-channel basis using ``sagemaker.session.s3_input.input_mode``.
-            output_path (str): S3 location for saving the training result (model artifacts and output files).
-                If not specified, results are stored to a default bucket. If the bucket with the specific name
-                does not exist, the estimator creates the bucket during the
-                :meth:`~sagemaker.estimator.EstimatorBase.fit` method execution.
-            output_kms_key (str): Optional. KMS key ID for encrypting the training output (default: None).
-            base_job_name (str): Prefix for training job name when the :meth:`~sagemaker.estimator.EstimatorBase.fit`
-                method launches. If not specified, the estimator generates a default job name, based on
-                the training image name and current timestamp.
-            sagemaker_session (sagemaker.session.Session): Session object which manages interactions with
-                Amazon SageMaker APIs and any other AWS services needed. If not specified, the estimator creates one
-                using the default AWS configuration chain.
+                This argument can be overriden on a per-channel basis using
+                ``sagemaker.session.s3_input.input_mode``.
+
+            output_path (str): S3 location for saving the training result (model artifacts and
+                output files). If not specified, results are stored to a default bucket. If
+                the bucket with the specific name does not exist, the
+                estimator creates the bucket during the
+                :meth:`~sagemaker.estimator.EstimatorBase.fit` method
+                execution.
+            output_kms_key (str): Optional. KMS key ID for encrypting the
+                training output (default: None). base_job_name (str): Prefix for
+                training job name when the
+                :meth:`~sagemaker.estimator.EstimatorBase.fit`
+                method launches. If not specified, the estimator generates a
+                default job name, based on the training image name and
+                current timestamp.
+            sagemaker_session (sagemaker.session.Session): Session object which manages
+                interactions with Amazon SageMaker APIs and any other AWS services needed. If
+                not specified, the estimator creates one using the default
+                AWS configuration chain.
             tags (list[dict]): List of tags for labeling a training job. For more, see
                 https://docs.aws.amazon.com/sagemaker/latest/dg/API_Tag.html.
-            subnets (list[str]): List of subnet ids. If not specified training job will be created without VPC config.
-            security_group_ids (list[str]): List of security group ids. If not specified training job will be created
-                without VPC config.
-            model_uri (str): URI where a pre-trained model is stored, either locally or in S3 (default: None). If
-                specified, the estimator will create a channel pointing to the model so the training job can download
-                it. This model can be a 'model.tar.gz' from a previous training job, or other artifacts coming from a
-                different source.
-                More information: https://docs.aws.amazon.com/sagemaker/latest/dg/cdf-training.html#td-deserialization
-            model_channel_name (str): Name of the channel where 'model_uri' will be downloaded (default: 'model').
-            metric_definitions (list[dict]): A list of dictionaries that defines the metric(s) used to evaluate the
-                training jobs. Each dictionary contains two keys: 'Name' for the name of the metric, and 'Regex' for
-                the regular expression used to extract the metric from the logs.
-            encrypt_inter_container_traffic (bool): Specifies whether traffic between training containers is encrypted
-                for the training job (default: ``False``).
+            subnets (list[str]): List of subnet ids. If not specified
+                training job will be created without VPC config.
+                security_group_ids (list[str]): List of security group ids. If
+                not specified training job will be created without VPC config.
+            model_uri (str): URI where a pre-trained model is stored, either locally or in S3
+                (default: None). If specified, the estimator will create a channel pointing to
+                the model so the training job can download it. This model
+                can be a 'model.tar.gz' from a previous training job, or
+                other artifacts coming from a different source.
+                More information:
+                https://docs.aws.amazon.com/sagemaker/latest/dg/cdf-training.html#td-deserialization
+            model_channel_name (str): Name of the channel where 'model_uri'
+                will be downloaded (default: 'model'). metric_definitions
+                (list[dict]): A list of dictionaries that defines the metric(s)
+                used to evaluate the training jobs. Each dictionary contains two keys: 'Name' for
+                the name of the metric, and 'Regex' for the regular
+                expression used to extract the metric from the logs.
+            encrypt_inter_container_traffic (bool): Specifies whether traffic between training
+                containers is encrypted for the training job (default: ``False``).
+            **kwargs: Additional kwargs. This is unused. It's only added for AlgorithmEstimator
+                to ignore the irrelevant arguments.
         """
         self.algorithm_arn = algorithm_arn
         super(AlgorithmEstimator, self).__init__(
@@ -137,6 +161,7 @@ class AlgorithmEstimator(EstimatorBase):
             self.set_hyperparameters(**hyperparameters)
 
     def validate_train_spec(self):
+        """Placeholder docstring"""
         train_spec = self.algorithm_spec["TrainingSpecification"]
         algorithm_name = self.algorithm_spec["AlgorithmName"]
 
@@ -169,6 +194,10 @@ class AlgorithmEstimator(EstimatorBase):
             )
 
     def set_hyperparameters(self, **kwargs):
+        """
+        Args:
+            **kwargs:
+        """
         for k, v in kwargs.items():
             value = self._validate_and_cast_hyperparameter(k, v)
             self.hyperparam_dict[k] = value
@@ -178,23 +207,25 @@ class AlgorithmEstimator(EstimatorBase):
     def hyperparameters(self):
         """Returns the hyperparameters as a dictionary to use for training.
 
-        The fit() method, that does the model training, calls this method to find the hyperparameters you specified.
+        The fit() method, that does the model training, calls this method to
+        find the hyperparameters you specified.
         """
         return self.hyperparam_dict
 
     def train_image(self):
         """Returns the docker image to use for training.
 
-        The fit() method, that does the model training, calls this method to find the image to use for model training.
+        The fit() method, that does the model training, calls this method to
+        find the image to use for model training.
         """
         raise RuntimeError("train_image is never meant to be called on Algorithm Estimators")
 
     def enable_network_isolation(self):
         """Return True if this Estimator will need network isolation to run.
 
-        On Algorithm Estimators this depends on the algorithm being used. If this is algorithm
-        owned by your account it will be False. If this is an an algorithm consumed from Marketplace
-        it will be True.
+        On Algorithm Estimators this depends on the algorithm being used. If
+        this is algorithm owned by your account it will be False. If this is an
+        an algorithm consumed from Marketplace it will be True.
 
         Returns:
             bool: Whether this Estimator needs network isolation or not.
@@ -214,25 +245,39 @@ class AlgorithmEstimator(EstimatorBase):
     ):
         """Create a model to deploy.
 
-        The serializer, deserializer, content_type, and accept arguments are only used to define a default
-        RealTimePredictor. They are ignored if an explicit predictor class is passed in. Other arguments
-        are passed through to the Model class.
+        The serializer, deserializer, content_type, and accept arguments are
+        only used to define a default RealTimePredictor. They are ignored if an
+        explicit predictor class is passed in. Other arguments are passed
+        through to the Model class.
 
         Args:
-            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also used during
-                transform jobs. If not specified, the role from the Estimator will be used.
-            predictor_cls (RealTimePredictor): The predictor class to use when deploying the model.
-            serializer (callable): Should accept a single argument, the input data, and return a sequence
-                of bytes. May provide a content_type attribute that defines the endpoint request content type
-            deserializer (callable): Should accept two arguments, the result data and the response content type,
-                and return a sequence of bytes. May provide a content_type attribute that defines th endpoint
-                response Accept content type.
-            content_type (str): The invocation ContentType, overriding any content_type from the serializer
-            accept (str): The invocation Accept, overriding any accept from the deserializer.
-            vpc_config_override (dict[str, list[str]]): Optional override for VpcConfig set on the model.
-                Default: use subnets and security groups from this Estimator.
+            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``,
+                which is also used during transform jobs. If not specified, the
+                role from the Estimator will be used.
+            predictor_cls (RealTimePredictor): The predictor class to use when
+                deploying the model.
+            serializer (callable): Should accept a single argument, the input
+                data, and return a sequence of bytes. May provide a content_type
+                attribute that defines the endpoint request content type
+            deserializer (callable): Should accept two arguments, the result
+                data and the response content type, and return a sequence of
+                bytes. May provide a content_type attribute that defines the
+                endpoint response Accept content type.
+            content_type (str): The invocation ContentType, overriding any
+                content_type from the serializer
+            accept (str): The invocation Accept, overriding any accept from the
+                deserializer.
+            vpc_config_override (dict[str, list[str]]): Optional override for VpcConfig set on
+                the model. Default: use subnets and security groups from this Estimator.
                 * 'Subnets' (list[str]): List of subnet ids.
                 * 'SecurityGroupIds' (list[str]): List of security group ids.
+            **kwargs: Additional arguments for creating a :class:`~sagemaker.model.ModelPackage`.
+
+        .. tip::
+
+            You can find additional parameters for using this method at
+            :class:`~sagemaker.model.ModelPackage` and
+            :class:`~sagemaker.model.Model`.
 
         Returns:
             a Model ready for deployment.
@@ -274,29 +319,40 @@ class AlgorithmEstimator(EstimatorBase):
         role=None,
         volume_kms_key=None,
     ):
-        """Return a ``Transformer`` that uses a SageMaker Model based on the training job. It reuses the
-        SageMaker Session and base job name used by the Estimator.
+        """Return a ``Transformer`` that uses a SageMaker Model based on the
+        training job. It reuses the SageMaker Session and base job name used by
+        the Estimator.
 
         Args:
             instance_count (int): Number of EC2 instances to use.
-            instance_type (str): Type of EC2 instance to use, for example, 'ml.c4.xlarge'.
-            strategy (str): The strategy used to decide how to batch records in a single request (default: None).
-                Valid values: 'MULTI_RECORD' and 'SINGLE_RECORD'.
-            assemble_with (str): How the output is assembled (default: None). Valid values: 'Line' or 'None'.
-            output_path (str): S3 location for saving the transform result. If not specified, results are stored to
-                a default bucket.
-            output_kms_key (str): Optional. KMS key ID for encrypting the transform output (default: None).
-            accept (str): The content type accepted by the endpoint deployed during the transform job.
-            env (dict): Environment variables to be set for use during the transform job (default: None).
-            max_concurrent_transforms (int): The maximum number of HTTP requests to be made to
-                each individual transform container at one time.
-            max_payload (int): Maximum size of the payload in a single HTTP request to the container in MB.
-            tags (list[dict]): List of tags for labeling a transform job. If none specified, then the tags used for
-                the training job are used for the transform job.
-            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``, which is also used during
-                transform jobs. If not specified, the role from the Estimator will be used.
-            volume_kms_key (str): Optional. KMS key ID for encrypting the volume attached to the ML
-                compute instance (default: None).
+            instance_type (str): Type of EC2 instance to use, for example,
+                'ml.c4.xlarge'.
+            strategy (str): The strategy used to decide how to batch records in
+                a single request (default: None). Valid values: 'MultiRecord'
+                and 'SingleRecord'.
+            assemble_with (str): How the output is assembled (default: None).
+                Valid values: 'Line' or 'None'.
+            output_path (str): S3 location for saving the transform result. If
+                not specified, results are stored to a default bucket.
+            output_kms_key (str): Optional. KMS key ID for encrypting the
+                transform output (default: None).
+            accept (str): The accept header passed by the client to
+                the inference endpoint. If it is supported by the endpoint,
+                it will be the format of the batch transform output.
+            env (dict): Environment variables to be set for use during the
+                transform job (default: None).
+            max_concurrent_transforms (int): The maximum number of HTTP requests
+                to be made to each individual transform container at one time.
+            max_payload (int): Maximum size of the payload in a single HTTP
+                request to the container in MB.
+            tags (list[dict]): List of tags for labeling a transform job. If
+                none specified, then the tags used for the training job are used
+                for the transform job.
+            role (str): The ``ExecutionRoleArn`` IAM Role ARN for the ``Model``,
+                which is also used during transform jobs. If not specified, the
+                role from the Estimator will be used.
+            volume_kms_key (str): Optional. KMS key ID for encrypting the volume
+                attached to the ML compute instance (default: None).
         """
         role = role or self.role
 
@@ -334,23 +390,39 @@ class AlgorithmEstimator(EstimatorBase):
         )
 
     def _is_marketplace(self):
+        """Placeholder docstring"""
         return "ProductId" in self.algorithm_spec
 
     def _prepare_for_training(self, job_name=None):
         # Validate hyperparameters
         # an explicit call to set_hyperparameters() will also validate the hyperparameters
         # but it is possible that the user never called it.
+        """
+        Args:
+            job_name:
+        """
         self._validate_and_set_default_hyperparameters()
 
         super(AlgorithmEstimator, self)._prepare_for_training(job_name)
 
     def fit(self, inputs=None, wait=True, logs=True, job_name=None):
+        """
+        Args:
+            inputs:
+            wait:
+            logs:
+            job_name:
+        """
         if inputs:
             self._validate_input_channels(inputs)
 
         super(AlgorithmEstimator, self).fit(inputs, wait, logs, job_name)
 
     def _validate_input_channels(self, channels):
+        """
+        Args:
+            channels:
+        """
         train_spec = self.algorithm_spec["TrainingSpecification"]
         algorithm_name = self.algorithm_spec["AlgorithmName"]
         training_channels = {c["Name"]: c for c in train_spec["TrainingChannels"]}
@@ -368,6 +440,11 @@ class AlgorithmEstimator(EstimatorBase):
                 raise ValueError("Required input channel: %s Was not provided." % (name))
 
     def _validate_and_cast_hyperparameter(self, name, v):
+        """
+        Args:
+            name:
+            v:
+        """
         algorithm_name = self.algorithm_spec["AlgorithmName"]
 
         if name not in self.hyperparameter_definitions:
@@ -387,6 +464,7 @@ class AlgorithmEstimator(EstimatorBase):
         return value
 
     def _validate_and_set_default_hyperparameters(self):
+        """Placeholder docstring"""
         # Check if all the required hyperparameters are set. If there is a default value
         # for one, set it.
         for name, definition in self.hyperparameter_definitions.items():
@@ -398,6 +476,7 @@ class AlgorithmEstimator(EstimatorBase):
                     raise ValueError("Required hyperparameter: %s is not set" % name)
 
     def _parse_hyperparameters(self):
+        """Placeholder docstring"""
         definitions = {}
 
         training_spec = self.algorithm_spec["TrainingSpecification"]
@@ -419,6 +498,11 @@ class AlgorithmEstimator(EstimatorBase):
         return definitions
 
     def _hyperparameter_range_and_class(self, parameter_type, hyperparameter):
+        """
+        Args:
+            parameter_type:
+            hyperparameter:
+        """
         if parameter_type in self._hyperpameters_with_range:
             range_name = parameter_type + "ParameterRangeSpecification"
 
@@ -459,9 +543,38 @@ class AlgorithmEstimator(EstimatorBase):
         return parameter_class, parameter_range
 
     def _algorithm_training_input_modes(self, training_channels):
+        """
+        Args:
+            training_channels:
+        """
         current_input_modes = {"File", "Pipe"}
         for channel in training_channels:
             supported_input_modes = set(channel["SupportedInputModes"])
             current_input_modes = current_input_modes & supported_input_modes
 
         return current_input_modes
+
+    @classmethod
+    def _prepare_init_params_from_job_description(cls, job_details, model_channel_name=None):
+        """Convert the job description to init params that can be handled by the
+        class constructor
+
+        Args:
+            job_details (dict): the returned job details from a DescribeTrainingJob
+                API call.
+            model_channel_name (str): Name of the channel where pre-trained
+                model data will be downloaded.
+
+        Returns:
+            dict: The transformed init_params
+        """
+        init_params = super(AlgorithmEstimator, cls)._prepare_init_params_from_job_description(
+            job_details, model_channel_name
+        )
+
+        # This hyperparameter is added by Amazon SageMaker Automatic Model Tuning.
+        # It cannot be set through instantiating an estimator.
+        if "_tuning_objective_metric" in init_params["hyperparameters"]:
+            del init_params["hyperparameters"]["_tuning_objective_metric"]
+
+        return init_params
