@@ -1,4 +1,4 @@
-# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2018-2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -10,11 +10,17 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+"""Placeholder docstring"""
 from __future__ import absolute_import
 
 PYTORCH_VERSION = "0.4"
 """Default PyTorch version for when the framework version is not specified.
-This is no longer updated so as to not break existing workflows.
+The default version is no longer updated so as to not break existing workflows.
 """
 
+LATEST_VERSION = "1.4.0"
+"""The latest version of PyTorch included in the SageMaker pre-built Docker images."""
+
 PYTHON_VERSION = "py3"
+
+LATEST_PY2_VERSION = "1.3.1"
