@@ -165,22 +165,27 @@ class SKLearn(Framework):
         # remove unwanted entry_point kwarg
         if "entry_point" in kwargs:
             logger.debug("removing unused entry_point argument: %s", str(kwargs["entry_point"]))
-            kwargs = {k: v for k, v in kwargs.items() if k != "entry_point"}
+            del kwargs["entry_point"]
 
         # remove image kwarg
         if "image" in kwargs:
             image = kwargs["image"]
-            kwargs = {k: v for k, v in kwargs.items() if k != "image"}
+            del kwargs["image"]
         else:
             image = None
 
+        if "enable_network_isolation" not in kwargs:
+            kwargs["enable_network_isolation"] = self.enable_network_isolation()
+
+        if "name" not in kwargs:
+            kwargs["name"] = self._current_job_name
+
         return SKLearnModel(
             self.model_data,
             role,
             self.entry_point,
             source_dir=self._model_source_dir(),
             enable_cloudwatch_metrics=self.enable_cloudwatch_metrics,
-            name=self._current_job_name,
             container_log_level=self.container_log_level,
             code_location=self.code_location,
             py_version=self.py_version,
@@ -189,7 +194,6 @@ class SKLearn(Framework):
             image=image or self.image_name,
             sagemaker_session=self.sagemaker_session,
             vpc_config=self.get_vpc_config(vpc_config_override),
-            enable_network_isolation=self.enable_network_isolation(),
             **kwargs
         )
 
